import _Object$keys from "babel-runtime/core-js/object/keys";
import _typeof from "babel-runtime/helpers/typeof";
import _extends from "babel-runtime/helpers/extends";
import _classCallCheck from "babel-runtime/helpers/classCallCheck";
import _possibleConstructorReturn from "babel-runtime/helpers/possibleConstructorReturn";
import _inherits from "babel-runtime/helpers/inherits";
import { PureComponent } from "react";
import PropTypes from "prop-types";
import axios from "axios";
import _debounce from "lodash.debounce";
import _isPlainObject from "lodash.isplainobject";
import _isEqual from "fast-deep-equal";
import { isEvent, serialize, concatParams, parseMime } from "./utils";

var defaultFetchCreator = function defaultFetchCreator(requestData) {
  return axios(requestData);
};

var AsyncFetcher = function (_PureComponent) {
  _inherits(AsyncFetcher, _PureComponent);

  function AsyncFetcher(props) {
    _classCallCheck(this, AsyncFetcher);

    var _this = _possibleConstructorReturn(this, _PureComponent.call(this, props));

    _this.fetch = function () {
      var dataEvent = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      var customRequest = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
      var _this$props = _this.props,
          url = _this$props.url,
          headers = _this$props.headers,
          axiosConfig = _this$props.axiosConfig,
          authToken = _this$props.authToken,
          ajax = _this$props.ajax,
          accepts = _this$props.accepts,
          responseType = _this$props.responseType,
          withCredentials = _this$props.withCredentials,
          transformError = _this$props.transformError,
          fetchCreator = _this$props.fetchCreator,
          onResponse = _this$props.onResponse,
          onError = _this$props.onError;


      var method = _this.props.method.toLowerCase();
      var isPost = method === "post" || method === "put";
      var isEventData = !!dataEvent && isEvent(dataEvent);

      var contentType = !!_this.props.contentType ? parseMime(_this.props.contentType) : isPost ? parseMime("form") : null;

      var customHeaders = {};
      if (!!contentType) {
        customHeaders["Content-Type"] = contentType;
      }
      if (!!authToken) {
        var authorization = typeof authToken === "function" ? authToken() : authToken;
        if (!!authorization) {
          if (authorization.indexOf("Bearer") === -1) {
            authorization = "Bearer " + authorization;
          }
          customHeaders["Authorization"] = authorization;
        }
      }
      if (!!ajax) {
        customHeaders["X-Requested-With"] = "XMLHttpRequest";
      }
      if (!!accepts) {
        customHeaders["Accept"] = function () {
          if (typeof accepts === "string") {
            return accepts.split(";").map(parseMime).join(";");
          }
          if (Array.isArray(accepts)) {
            return accepts.map(parseMime).join(";");
          }
          if (typeof accepts === "function") {
            return accepts();
          }
          return accepts;
        }();
      }

      var requestConfig = {
        method: method,
        withCredentials: !!withCredentials,
        headers: _extends({}, headers, customHeaders)
      };

      var params = !isPost && !!dataEvent && !isEventData ? dataEvent : concatParams(_this.props.params, _this.state.params);

      if (typeof url === "function") {
        requestConfig.url = url(!!params ? params : {});
      } else {
        requestConfig.url = url;

        if (!!params) {
          if (typeof params !== "string" && _isPlainObject(params)) {
            params = serialize(params);
          }
          if (typeof params === "string") {
            requestConfig.url = (requestConfig.url + (requestConfig.url.indexOf("?") === -1 ? "?" : "&") + params).replace(/\?&/gi, "?");
          } else {
            requestConfig.params = params;
          }
        }
      }

      if (isPost) {
        var data = !!dataEvent && !isEventData ? dataEvent : concatParams(_this.props.postData, _this.state.postData);
        if (!!data && contentType.indexOf("x-www-form-urlencoded") !== -1 && _isPlainObject(data)) {
          data = serialize(data);
        }
        requestConfig.data = data;
      }

      if (!!responseType) {
        requestConfig.responseType = responseType;
      }

      _this.setState({ isLoading: true, error: null }, function () {
        return fetchCreator(_extends({}, requestConfig, axiosConfig, customRequest)).then(function (response) {
          if (!_this._mounted) {
            return;
          }

          _this.setState({
            response: response,
            data: response.data,
            isLoading: false
          }, function () {
            if (!!onResponse) onResponse(response);
          });
        }).catch(function (error) {
          if (!_this._mounted) {
            return;
          }

          var transformedError = transformError(_extends({}, error));

          _this.setState(function () {
            var errorState = {
              error: "" + (!!transformedError && typeof transformedError === "string" ? transformedError : error),
              response: null,
              data: null,
              isLoading: false
            };
            if (!!transformedError && (typeof transformedError === "undefined" ? "undefined" : _typeof(transformedError)) === "object" && "response" in transformedError) {
              errorState.response = transformedError.response;

              if (!!errorState.response && _typeof(errorState.response) === "object" && "data" in errorState.response) {
                errorState.data = errorState.response.data;
              }
            }

            return errorState;
          }, function () {
            if (!!onError) onError(transformedError);
          });
        });
      });
    };

    _this.setCustomState = function (state) {
      if (!!state && ((typeof state === "undefined" ? "undefined" : _typeof(state)) === "object" || typeof state === "function")) {
        var hasCustom = false;
        var customState = _this.state.customState;

        if (typeof state === "function") {
          state = state(_extends({
            params: _this.state.params,
            postData: _this.state.postData
          }, customState));
          if ((typeof state === "undefined" ? "undefined" : _typeof(state)) !== "object") {
            return;
          }
        }

        _Object$keys(state).forEach(function (k) {
          if (["params", "postData", "data"].indexOf(k) !== -1) {
            if (!_isEqual(_this.state[k], state[k])) {
              var _this$setState;

              _this.setState((_this$setState = {}, _this$setState[k] = state[k], _this$setState));
            }
          } else {
            customState[k] = state[k];
            hasCustom = true;
          }
        });

        if (hasCustom) {
          _this.setState({ customState: customState });
        }
      }
    };

    _this._mounted = false;


    _this._debouncedFetch = +props.debounce > 0 ? _debounce(_this.fetch.bind(_this), +props.debounce) : _this.fetch.bind(_this);

    _this.state = {
      isLoading: !!props.autoFetch,
      response: null,
      data: null,
      error: null,
      params: null,
      postData: null,
      customState: {}
    };
    return _this;
  }

  AsyncFetcher.prototype.componentDidUpdate = function componentDidUpdate(prevProps, prevState) {
    var _this2 = this;

    var hasChange = false;
    ["method", "params", "postData", "url", "authToken"].forEach(function (k) {
      if (k in prevProps && !_isEqual(prevProps[k], _this2.props[k])) {
        hasChange = true;
      }
    });
    if (!hasChange) {
      ["params", "postData"].forEach(function (k) {
        if (k in prevState && !_isEqual(prevState[k], _this2.state[k])) {
          hasChange = true;
        }
      });
    }

    if (+prevProps.debounce !== +this.props.debounce) {
      this._debouncedFetch = +this.props.debounce > 0 ? _debounce(this.fetch.bind(this), +this.props.debounce) : this.fetch.bind(this);
    }

    if (hasChange && !!this.props.autoFetch) {
      this._debouncedFetch();
    }
  };

  AsyncFetcher.prototype.componentDidMount = function componentDidMount() {
    this._mounted = true;

    if (!!this.props.autoFetch) {
      this.fetch();
    }
  };

  AsyncFetcher.prototype.componentWillUnmount = function componentWillUnmount() {
    this._mounted = false;
  };

  AsyncFetcher.prototype.render = function render() {
    if (!!this.props.children && typeof this.props.children === "function") {
      return this.props.children({
        fetch: this.fetch,
        isLoading: this.state.isLoading,
        response: this.state.response,
        data: this.state.data,
        error: this.state.error,
        params: concatParams(this.props.params, this.state.params),
        postData: concatParams(this.props.postData, this.state.postData),
        state: this.state.customState,
        set: this.setCustomState
      }) || null;
    }

    return null;
  };

  return AsyncFetcher;
}(PureComponent);

AsyncFetcher.propTypes = {
  autoFetch: PropTypes.bool,
  method: PropTypes.oneOf(["get", "post", "put", "delete", "options"]),
  url: PropTypes.oneOfType([PropTypes.string, PropTypes.func]).isRequired,
  params: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
  postData: PropTypes.oneOfType([PropTypes.string, PropTypes.object]),
  headers: PropTypes.object,
  axiosConfig: PropTypes.object,
  ajax: PropTypes.bool,
  authToken: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),
  accepts: PropTypes.oneOfType([PropTypes.string, PropTypes.array, PropTypes.func]),
  contentType: PropTypes.string,
  responseType: PropTypes.string,
  withCredentials: PropTypes.bool,
  transformError: PropTypes.func,
  fetchCreator: PropTypes.func,
  onResponse: PropTypes.func,
  onError: PropTypes.func,
  debounce: PropTypes.number,
  children: PropTypes.func
};
AsyncFetcher.defaultProps = {
  autoFetch: true,
  method: "get",
  headers: {},
  axiosConfig: {},
  ajax: false,
  withCredentials: false,
  transformError: function transformError(err) {
    return err;
  },
  fetchCreator: defaultFetchCreator,
  debounce: 0
};
export default AsyncFetcher;